from abc import ABC, abstractmethod

from immutable_views import DictView


class MessageArchitectIF(ABC):
    """
    Generic methods to build AMQP Messages
    """

    @staticmethod
    @abstractmethod
    def get_message_template(msg_type: str) -> DictView:
        """
        Retrieves a DictView of the template for the message type provided
        :param msg_type: type of message to send
        :return: template DictView
        """
        raise NotImplementedError

    @abstractmethod
    def compose_message(self, msg_type: str) -> DictView:
        """
        Uses a templated message to construct a real AMQP message and returns it
        :param msg_type: type of message to send
        :return: message DictView
        """
        raise NotImplementedError
